@@ -11,7 +11,7 @@ import ctypes
 import os
 import fnmatch
 
-dir_path = os.path.dirname(os.path.realpath(__file__)) + os.path.sep + '..' + os.path.sep
+dir_path = os.path.dirname(os.path.realpath(__file__))
 fileNameTemplate = 'RGFLib*.so'
 flist = fnmatch.filter(os.listdir(dir_path + os.path.sep), fileNameTemplate)
 RGFCLib = ctypes.cdll.LoadLibrary(dir_path + os.path.sep + flist[0])
